#!/usr/bin/perl

#Murasaki - multiple genome global alignment program
#Copyright (C) 2006-2007 Keio University
#(Kris Popendorf) <comp@bio.keio.ac.jp> (2006)
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

use strict;
use Term::ReadKey;
use Text::Wrap;

$Text::Wrap::columns=72;
abort("Please run from the Murasaki home directory") unless (-f "src/murasaki.cc" or -x "murasaki");
our $pwd=$ENV{PWD};

#add to .bashrc unelss we're already there
our $shellrc="$ENV{HOME}/.bashrc";
$shellrc="$ENV{HOME}/.cshrc" if $ENV{SHELL}=~m/csh$/;
if($ENV{MURASAKI} ne $pwd or !(`cat $shellrc`=~m/^export MURASAKI/m)){
  if($ENV{MURASAKI}){
    if($ENV{MURASAKI} ne $pwd){
      wprint("You have a MURASAKI environment variable set, but not for this directory...\n".
	     "I'm not going to try to erase old values from your $shellrc (for fear of erasing something important by accident), we recommend you do that yourself.\n".
	    "However, simply appending a new setting wouldn't hurt.\n");
    }else{
      wprint("Your MURASAKI environment variable is set, but it doesn't appear ".
	   "to be set in $shellrc.\n");
    }
  }else{
    print "It looks like you don't have a MURASAKI environment variable set.\n";
    print "This should be set to $pwd\n";
  }
  if($ENV{SHELL}=~m/sh$/){
    do {
      $_=prompt(wrap('','',"Your shell ($ENV{SHELL}) probably allows us to set this in ".
		"$shellrc. Would you like me to set this for you? [Y/n] "));
      if(m/y|\n/im){
	if($ENV{SHELL}=~m/csh$/){
	  system("echo setenv MURASAKI $pwd >> $shellrc");
	}else{
	  system("echo export MURASAKI=\"$pwd\" >> $shellrc");
	}
	wprint("Ok. Set.\nYou might want to 'source $shellrc' to enable it now.\n");
      }
    }until(m/y|n|\n/im);
  }else{
    wprint("Your shell ($ENV{SHELL}) is not supported, so you'll have to".
      "set that variable manually. Sorry...\n");
  }
}

do{
  if(-e "config.pl"){
    $_=prompt("config.pl already exists. Do you want to overwrite it? [y/N] ");
    if(m/^n|\n/im){
      wprint("config.pl not overwritten.\n");
      goto DONE;
    }
  }
  else{$_="y"}
  print "Writing a new config.pl\n";

  open(OUTF,">config.pl") or die "Could not write to config.pl";
print OUTF <<ENDTEXT;
#!/usr/bin/perl
#config file for the murasaki perl scripts

our \$root='$pwd';
our \$seqhome="\$root/seq";
ENDTEXT
}until(m/^y|n|\n/im);

DONE:
wprint("Setup finished!\n");
exit(0);

sub abort(){
  print join("\n",@_)."\nAborting...\n";
  exit(1);
}

sub prompt(){
  my $buffered=$|;
  ReadMode 3; #cbreak mode
  $|=1;
  print @_;
  my $key;
  while (not defined ($key = ReadKey(1))) {
    # No key yet
  }
  print $key;
  print "\n" unless $key=~m/\n$/m;
  ReadMode 0;
  $|=$buffered;
  return $key;
}

sub wprint(){
  print wrap('','',@_);
}
